clear ;
%
%  CAR WITH 2 TRAILERS PACKING VIA FLATNESS AND FRENET FORMULAS
%
%   explicit computation and visualisation of the motions.
%
%    February 1993
%
% ............................................................
% :         Pierre ROUCHON  <rouchon@cas.ensmp.fr>           :
% : Centre Automatique et Systemes, Ecole des Mines de Paris :
% : 60, bd Saint Michel -- 75272 PARIS CEDEX 06, France      :
% :    Telephone: (1) 40 51 91 15 --- Fax: (1) 43 54 18 93   :
% :..........................................................:
%
% lengths 
%    L:  car length (m) 
%   d1: trailer 1 length (m)
%   d2: trailer 2 length (m)
% T: basic time interval for one  smooth motion (s)
% a0, a1, b0, p(5): intermediate variables for polynomial 
%           curves definition
%
 global T L d1 d2  a0 a1 b0 p  ;
%
clc ;
T = 1 ; 
L = 1 ; d1 = 1.5 ; d2 = 1 ; 
a0 =0 ; a1 = 0 ; b0 = 0 ;
p= [0 0 0 0 0 ] ;
%
% initial configuration
%   the system is described via the coordinates of last trailer
%    
 %x2_1 = -2 ; y2_1 = 5 ;  theta2_1= pi/4; theta12_1 =  0 ; theta01_1= 0; phi_1 = 0 ;
 x2_1 = 0 ; y2_1 = 3 ;  theta2_1= 0; theta12_1 =  0 ; theta01_1= 0; phi_1 = 0 ;
% x2_1 = 0 ; y2_1 = 1 ;  theta2_1= 0; theta12_1 =  0 ; theta01_1= 0; phi_1 = 0 ;
%
% final configuration of the car 
x2_2 = 0 ; y2_2 = 0  ;
theta2_2= 0 ; theta12_2 = 0 ; theta01_2= 0 ;
                                   phi_2 = 0 ;
%
% sampling of motion 1 --> 0 and of motion 0 --> 2
        nbpt1 = 1000 ; nbpt2 = 1000 ;
%
% Constraints: y2_1 > y2_2 and 
% the 4 angles  theta2_1,2
%               theta12_1,2
%               theta01_1,2
%               phi_1,2 
%   must belong to  ] -pi/2 , + pi/2 [
%               
%
% conputation of  intermediate configuration
LL=L+d1+d2
x2_0 = max(x2_1,x2_2) ....
      + LL*abs(tan(theta2_1)) ....
      + LL*abs(tan(theta12_1)) .... 
      + LL*abs(tan(theta01_1)) .... 
      + LL*( abs(y2_1-y2_2)/(d1+d2+L) )^(1/2) ;
y2_0 = (y2_1+y2_2)/2 ;
%
%
% first polynomial curve
a0 = x2_0 ; b0 = y2_0 ;
a1 = x2_1 ; b1 = y2_1 ; 
p=car2Tkf((b1-b0),theta2_1,theta12_1,theta01_1,phi_1) ;
%
% computation the first motion  1 -> 0
 theta2 = theta2_1 ;
 theta1 = theta12_1+theta2 ;
 theta0 = theta01_1+theta1 ;
 phi = phi_1 ;
 x0=x2_1+d2*cos(theta2)+d1*cos(theta1) ;
 y0=y2_1+d2*sin(theta2)+d1*sin(theta1) ;
 state_1 = [x0 y0 theta0 theta1 theta2 phi] ;
 for i=1:(nbpt1+1)
   tau = (i-1)/nbpt1 ;
   phi=tau*tau*(3-2*tau) ;
   aa = (1-phi)*a1 + phi*a0 ;
   [bb df d2f d3f d4f d5f] = car2Tfjet(aa) ;
   [k2 k1 k0 dk0]=car2Tfk(df,d2f,d3f,d4f,d5f) ;
   theta2 = atan(df);
   theta1 = atan(k2*d2)+theta2;
   theta0 = atan(k1*d1) + theta1 ;
   phi = atan(k0*L) ;
   x0=aa+d2*cos(theta2)+d1*cos(theta1) ;
   y0=bb+d2*sin(theta2)+d1*sin(theta1) ;
   state_1 = [ state_1 ; x0 y0 theta0 theta1 theta2 phi] ;
 end ;
%
% second polynomial curve
a0 = x2_0 ; b0 = y2_0 ;
a1 = x2_2 ; b1 = y2_2 ; 
 p=car2Tkf((b1-b0),theta2_2,theta12_2,theta01_2,phi_2) ;
%
% computation of the second motion  0 -> 2
 theta2 = 0 ;
 theta1 = 0 ;
 theta0 = 0 ;
 phi = 0 ;
 x0=x2_0+d2*cos(theta2)+d1*cos(theta1) ;
 y0=y2_0+d2*sin(theta2)+d1*sin(theta1) ;
 state_2 = [x0 y0 theta0 theta1 theta2 phi] ;
 for i=1:(nbpt2+1)
   tau = (i-1)/nbpt2 ;
   phi=tau*tau*(3-2*tau) ;
   aa = (1-phi)*a0 + phi*a1 ;
   [bb df d2f d3f d4f d5f] = car2Tfjet(aa) ;
   [k2 k1 k0 dk0]=car2Tfk(df,d2f,d3f,d4f,d5f) ;
   theta2 = atan(df);
   theta1 = atan(k2*d2)+theta2;
   theta0 = atan(k1*d1) + theta1 ;
   phi = atan(k0*L) ;
   x0=aa+d2*cos(theta2)+d1*cos(theta1) ;
   y0=bb+d2*sin(theta2)+d1*sin(theta1) ;
   state_2 = [ state_2 ; x0 y0 theta0 theta1 theta2 phi] ;
 end ;
%
%
%
%
% Graphics 
%
% window size
xmin = min([min(state_1(:,1)) min(state_2(:,1))]) -1.5*(d1+d2)  ;
xmax = max([max(state_1(:,1)) max(state_1(:,1))]) +1.5*L ;
ymin = min([min(state_1(:,2)) min(state_2(:,2))])-L;
ymax = max([max(state_1(:,2)) max(state_1(:,2))])+L;
dd = max([xmax-xmin ymax-ymin]) ;

figure(1),clf;
set(gca,'position',[0 0 1 1],'visible','off','Xlim',[xmin xmin+dd],'Ylim',[ymin ymin+dd]....
   ,'nextplot','add');
axis square;
set(gcf,'renderer','openGL');
%
% starting configuration
   x2=x2_1 ; y2=y2_1 ;
   theta2 = theta2_1 ;
   theta1 = theta12_1+theta2 ;
   theta0 = theta01_1+theta1 ;
   phi = phi_1 ;
   x1=x2+d2*cos(theta2) ;
   y1=y2+d2*sin(theta2) ;
   x0=x1+d1*cos(theta1) ;
   y0=y1+d1*sin(theta1) ;
plotcar2T(x0,y0,theta0,theta1,theta2,phi,'-') ;
pause ;
%
% end configuration
   x2=x2_2 ; y2=y2_2 ;
   theta2 = theta2_2 ;
   theta1 = theta12_2+theta2 ;
   theta0 = theta01_2+theta1 ;
   phi = phi_2 ;
   x1=x2+d2*cos(theta2) ;
   y1=y2+d2*sin(theta2) ;
   x0=x1+d1*cos(theta1) ;
   y0=y1+d1*sin(theta1) ;
plotcar2T(x0,y0,theta0,theta1,theta2,phi,'--') ;
pause ; 
%
% intermediate configuration (inversion of velocity)
   x2=x2_0 ; y2=y2_0 ;
   theta2 = 0 ;
   theta1 = 0;
   theta0 = 0 ;
   phi = 0;
   x1=x2+d2*cos(theta2) ;
   y1=y2+d2*sin(theta2) ;
   x0=x1+d1*cos(theta1) ;
   y0=y1+d1*sin(theta1) ;
plotcar2T(x0,y0,theta0,theta1,theta2,phi,'--') ;
state_1 =[state_1;state_2] ;
x_lin = state_1(:,1)-d1*cos(state_1(:,4))-d2*cos(state_1(:,5)) ;
y_lin = state_1(:,2)-d1*sin(state_1(:,4))-d2*sin(state_1(:,5)) ;
pause;
%
%
%  motion
%
% trajectory of the linearizing output
plot(x_lin,y_lin,'linewidth',2)
pause ;
%
% movies 
[n m] = size(state_1) ;
for j=1:n
  h=plotcar2T(state_1(j,1),state_1(j,2),state_1(j,3), ....
      state_1(j,4),state_1(j,5),state_1(j,6),'-' ) ;
     drawnow;
 if (j<n)
  delete(h);
  end ;
end ;
 

